import _regeneratorRuntime from "@babel/runtime/regenerator";
import mqtt from 'mqtt';
import getMqttTopic from './get_mqtt_topic';
import { subscribeAdapterNotInitialized, subscribeAdapterAlreadyInitialized } from './constants';
var qos = 1;

var createClientAdapter = function createClientAdapter(_ref) {
  var url = _ref.url,
      appId = _ref.appId,
      onEvent = _ref.onEvent;
  var client;
  var isInitialized;
  return {
    init: function init() {
      return _regeneratorRuntime.async(function init$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!isInitialized) {
                _context.next = 2;
                break;
              }

              throw new Error(subscribeAdapterAlreadyInitialized);

            case 2:
              _context.next = 4;
              return _regeneratorRuntime.awrap(new Promise(function (resolve, reject) {
                client = mqtt.connect(url);
                client.on('connect', function () {
                  isInitialized = true;
                  resolve();
                });
                client.on('error', function (err) {
                  reject(err);
                });
                client.on('message', function (topic, message) {
                  try {
                    var event = JSON.parse(message.toString('utf8'));
                    onEvent(event);
                  } catch (error) {
                    // eslint-disable-next-line no-console
                    console.warn(topic, message, error);
                  }
                });
              }));

            case 4:
              return _context.abrupt("return", _context.sent);

            case 5:
            case "end":
              return _context.stop();
          }
        }
      });
    },
    close: function close() {
      return _regeneratorRuntime.async(function close$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (isInitialized) {
                _context2.next = 2;
                break;
              }

              throw new Error(subscribeAdapterNotInitialized);

            case 2:
              isInitialized = false;
              client.end();
              client = undefined;

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      });
    },
    subscribeToTopics: function subscribeToTopics(topics) {
      return _regeneratorRuntime.async(function subscribeToTopics$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (isInitialized) {
                _context3.next = 2;
                break;
              }

              throw new Error(subscribeAdapterNotInitialized);

            case 2:
              _context3.next = 4;
              return _regeneratorRuntime.awrap(Promise.all(topics.map(function (topic) {
                return new Promise(function (resolve, reject) {
                  return client.subscribe(getMqttTopic(appId, topic), {
                    qos: qos
                  }, function (err) {
                    return err ? reject(err) : resolve();
                  });
                });
              })));

            case 4:
            case "end":
              return _context3.stop();
          }
        }
      });
    },
    unsubscribeFromTopics: function unsubscribeFromTopics(topics) {
      return _regeneratorRuntime.async(function unsubscribeFromTopics$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (isInitialized) {
                _context4.next = 2;
                break;
              }

              throw new Error(subscribeAdapterNotInitialized);

            case 2:
              _context4.next = 4;
              return _regeneratorRuntime.awrap(Promise.all(topics.map(function (topic) {
                return new Promise(function (resolve, reject) {
                  return client.unsubscribe(getMqttTopic(appId, topic), function (err) {
                    return err ? reject(err) : resolve();
                  });
                });
              })));

            case 4:
            case "end":
              return _context4.stop();
          }
        }
      });
    },
    isConnected: function isConnected() {
      if (!isInitialized) {
        throw new Error(subscribeAdapterNotInitialized);
      }

      return client.connected;
    }
  };
};

createClientAdapter.adapterName = 'mqtt';
export default createClientAdapter;
//# sourceMappingURL=index.js.map