"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports["default"] = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _mqtt = _interopRequireDefault(require("mqtt"));

var _get_mqtt_topic = _interopRequireDefault(require("./get_mqtt_topic"));

var _constants = require("./constants");

var qos = 1;

var createClientAdapter = function createClientAdapter(_ref) {
  var url = _ref.url,
      appId = _ref.appId,
      onEvent = _ref.onEvent;
  var client;
  var isInitialized;
  return {
    init: function init() {
      return _regenerator["default"].async(function init$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!isInitialized) {
                _context.next = 2;
                break;
              }

              throw new Error(_constants.subscribeAdapterAlreadyInitialized);

            case 2:
              _context.next = 4;
              return _regenerator["default"].awrap(new Promise(function (resolve, reject) {
                client = _mqtt["default"].connect(url);
                client.on('connect', function () {
                  isInitialized = true;
                  resolve();
                });
                client.on('error', function (err) {
                  reject(err);
                });
                client.on('message', function (topic, message) {
                  try {
                    var event = JSON.parse(message.toString('utf8'));
                    onEvent(event);
                  } catch (error) {
                    // eslint-disable-next-line no-console
                    console.warn(topic, message, error);
                  }
                });
              }));

            case 4:
              return _context.abrupt("return", _context.sent);

            case 5:
            case "end":
              return _context.stop();
          }
        }
      });
    },
    close: function close() {
      return _regenerator["default"].async(function close$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              if (isInitialized) {
                _context2.next = 2;
                break;
              }

              throw new Error(_constants.subscribeAdapterNotInitialized);

            case 2:
              isInitialized = false;
              client.end();
              client = undefined;

            case 5:
            case "end":
              return _context2.stop();
          }
        }
      });
    },
    subscribeToTopics: function subscribeToTopics(topics) {
      return _regenerator["default"].async(function subscribeToTopics$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              if (isInitialized) {
                _context3.next = 2;
                break;
              }

              throw new Error(_constants.subscribeAdapterNotInitialized);

            case 2:
              _context3.next = 4;
              return _regenerator["default"].awrap(Promise.all(topics.map(function (topic) {
                return new Promise(function (resolve, reject) {
                  return client.subscribe((0, _get_mqtt_topic["default"])(appId, topic), {
                    qos: qos
                  }, function (err) {
                    return err ? reject(err) : resolve();
                  });
                });
              })));

            case 4:
            case "end":
              return _context3.stop();
          }
        }
      });
    },
    unsubscribeFromTopics: function unsubscribeFromTopics(topics) {
      return _regenerator["default"].async(function unsubscribeFromTopics$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              if (isInitialized) {
                _context4.next = 2;
                break;
              }

              throw new Error(_constants.subscribeAdapterNotInitialized);

            case 2:
              _context4.next = 4;
              return _regenerator["default"].awrap(Promise.all(topics.map(function (topic) {
                return new Promise(function (resolve, reject) {
                  return client.unsubscribe((0, _get_mqtt_topic["default"])(appId, topic), function (err) {
                    return err ? reject(err) : resolve();
                  });
                });
              })));

            case 4:
            case "end":
              return _context4.stop();
          }
        }
      });
    },
    isConnected: function isConnected() {
      if (!isInitialized) {
        throw new Error(_constants.subscribeAdapterNotInitialized);
      }

      return client.connected;
    }
  };
};

createClientAdapter.adapterName = 'mqtt';
var _default = createClientAdapter;
exports["default"] = _default;
//# sourceMappingURL=index.js.map